'use strict';

var react = require('react');
var reactDom = require('react-dom');

const mount = (options = {}) => {
  const { element = "div" } = options;
  return (Component) => {
    const componentName = Component.displayName || Component.name;
    return (attributes) => {
      let container = window.document.createElement(element);
      componentName && container.setAttribute("data-is", componentName);
      window.document.body.appendChild(container);
      const destroy = () => {
        if (!container) {
          return;
        }
        reactDom.unmountComponentAtNode(container);
        window.document.body.removeChild(container);
        container = null;
      };
      const promise = new Promise((resolve, reject) => {
        reactDom.render(react.createElement(Component, {
          ...attributes || {},
          destroy,
          resolve,
          reject
        }), container);
      });
      Object.defineProperty(promise, "destroy", {
        value: destroy,
        writable: false,
        enumerable: false,
        configurable: false
      });
      return promise;
    };
  };
};

module.exports = mount;
